isType   = require("@rill/is-type")
{ goTo } = require("./hijackBrowser") if process.browser
htmlReg  = /^\s*</
statuses =
	empty:
		204: true
		205: true
		304: true
	redirect:
		300: true
		301: true
		302: true
		303: true
		305: true
		307: true
		308: true

###
# Calculate bytelengths for complex strings.
#
# @param {String} str
# @return {Number}
###
byteLength = (str)->
	return 0 unless str
	str = str.toString()
	len = str.length
	for char in str
		code = char.charCodeAt()
		if 0x7f < code <= 0x7ff
			len += 1
		else if 0x7ff < code <= 0xffff
			len += 2
		else if 0xDC00 <= code <= 0xDFFF
			len -= 1
	return len


###
# A default middleware that handles sending the response for the request.
#
# @param {Object} req
# @param {Object} res
# @param {Function} next
###
module.exports = (req, res, next)->
	yield next
	{ method, redirectURL } = req
	{ res: originalRes, headers, status, body } = res

	# Skip request if something had already ended it.
	return if originalRes.headersSent

	# Handle redirects from the request object.
	if redirectURL?
		if process.browser
			return goTo(redirectURL,
				headers: headers
				method: method
			)
		else
			headers["location"] ?= redirectURL
			status = 302 unless statuses.redirect[status]

	# Default the status to 200 if there is substance to the response.
	status = 200 if Number(status) is 404 and body?

	if statuses.empty[status] or not body?
		headers["content-type"] ?= undefined
		body = undefined

	else if isType.String(body)
		headers["content-type"] ?= "text/#{if htmlReg.test(body) then "html" else "plain"}"

	else if isType.Buffer(body)
		headers["content-length"] = body.length

	else if isType.Stream(body)
		headers["content-type"] ?= "application/octet-stream"
		body.pipe(originalRes) unless process.browser
		return

	else
		headers["content-type"] ?= "application/json"
		try body = JSON.stringify(body)

	headers["content-length"] ?= byteLength(body)

	unless process.browser
		originalRes.writeHead(status, headers)
		originalRes.end(unless "HEAD" is method then body)
